#!/usr/bin/env bash

# Script to back up zookeeper data to S3
# This script should be run as cron job

# Create env file
# Needed to get access to AWS keys env variables from cron
source /opt/zookeeper/bin/env.sh

HOST=`hostname -s`
DOMAIN=`hostname -d`
DATE=`date -u '+%Y-%m-%dT%H:%M:%SZ'`
ZK_LOG_DIR=${ZK_LOG_DIR:-"/var/log/zookeeper"}
ZK_DATA_DIR=${ZK_DATA_DIR:-"/var/lib/zookeeper/data"}
ZK_DATA_LOG_DIR=${ZK_DATA_LOG_DIR:-"/var/lib/zookeeper/log"}
S3_BUCKET_NAME=${S3_BUCKET_NAME:-"cmbu-zk-backup"}
LOG_FILE=$ZK_LOG_DIR/zk-backup.log
S3_BASE_PATH=$S3_BUCKET_NAME/$DEPLOYMENT_ENVIRONMENT/$HOSTNAME

# Check if file size is greater 100MB, if yes delete it
# https://coderwall.com/p/pdkkxq/how-to-check-in-bash-if-file-size-is-greater-than-n
if [[ $(find $LOG_FILE -type f -size +100000000c 2>/dev/null) ]]; then
    rm -f $LOG_FILE
fi

echo "$DATE Starting backup of zookeeper data for $HOST" >> $LOG_FILE 2>&1

# Remove existing backup - avoid filling up local directory and also S3
rm -f /var/lib/zookeeper/backup*.tar.gz

# Create tar of snapshot directory
SRCDIR=$ZK_DATA_DIR
FILENAME=backup-$HOST-snapshot-$DATE.tar.gz
DESTDIR=/var/lib/zookeeper
tar -cpzf $DESTDIR/$FILENAME $SRCDIR
FILESIZE=$(stat -c%s "$DESTDIR/$FILENAME")
echo "$DATE Created tar of snapshot directory for $HOST at $DESTDIR/$FILENAME ($FILESIZE)" >> $LOG_FILE 2>&1
timeout --preserve-status 60 aws s3 cp $DESTDIR/$FILENAME s3://$S3_BASE_PATH/ >> $LOG_FILE 2>&1
aws_upload_status=$?
if [ ! $? -eq 0 ]; then
    echo "$DATE Failed to upload snapshot backup $DESTDIR/$FILENAME ($FILESIZE) for $HOST to S3 bucket $S3_BUCKET_NAME" >> $LOG_FILE 2>&1
else
    echo "$DATE Successfully uploaded snapshot backup $DESTDIR/$FILENAME ($FILESIZE) for $HOST to S3 bucket $S3_BUCKET_NAME" >> $LOG_FILE 2>&1
fi

# Create tar of tx-log directory
SRCDIR=$ZK_DATA_LOG_DIR
FILENAME=backup-$HOST-txlog-$DATE.tar.gz
DESTDIR=/var/lib/zookeeper
tar -cpzf $DESTDIR/$FILENAME $SRCDIR
FILESIZE=$(stat -c%s "$DESTDIR/$FILENAME")
echo "$DATE Created tar of tx-log directory for $HOST at $DESTDIR/$FILENAME ($FILESIZE)" >> $LOG_FILE 2>&1
timeout --preserve-status 60 aws s3 cp $DESTDIR/$FILENAME s3://$S3_BASE_PATH/ >> $LOG_FILE 2>&1
aws_upload_status=$?
if [ ! $? -eq 0 ]; then
    echo "$DATE Failed to upload tx-log backup $DESTDIR/$FILENAME ($FILESIZE) for $HOST to S3 bucket $S3_BUCKET_NAME" >> $LOG_FILE 2>&1
else
    echo "$DATE Successfully uploaded tx-log backup $DESTDIR/$FILENAME ($FILESIZE) for $HOST to S3 bucket $S3_BUCKET_NAME" >> $LOG_FILE 2>&1
fi

echo "$DATE Finished backup of zookeeper data for $HOST" >> $LOG_FILE 2>&1