#!/usr/bin/env bash
# Copyright 2016 The Kubernetes Authors.
#
# Licensed under the Apache License, Version 2.0 (the "License");
# you may not use this file except in compliance with the License.
# You may obtain a copy of the License at
#
#     http://www.apache.org/licenses/LICENSE-2.0
#
# Unless required by applicable law or agreed to in writing, software
# distributed under the License is distributed on an "AS IS" BASIS,
# WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.
# See the License for the specific language governing permissions and
# limitations under the License.

# zkOk.sh uses the ruok ZooKeeper four letter work to determine if the instance
# is health. The $? variable will be set to 0 if server responds that it is
# healthy, or 1 if the server fails to respond.

HOST=`hostname -s`
DOMAIN=`hostname -d`
DATE=`date -u '+%Y-%m-%dT%H:%M:%SZ'`
LOG_FILE=$ZK_LOG_DIR/zk-ready-live.log

# Check if file size is greater 200MB, if yes delete it
# https://coderwall.com/p/pdkkxq/how-to-check-in-bash-if-file-size-is-greater-than-n
if [[ $(find $LOG_FILE -type f -size +200000000c 2>/dev/null) ]]; then
    rm -f $LOG_FILE
fi

echo "$DATE Running readiness or liveness probe for $HOST" >> $LOG_FILE 2>&1

if [[ $HOST =~ (.*)-([0-9]+)$ ]]; then
    NAME=${BASH_REMATCH[1]}
    ORD=${BASH_REMATCH[2]}
else
    echo "$DATE Failed to extract ordinal from hostname $HOST" >> $LOG_FILE 2>&1
    exit 1
fi

for (( i=1; i<=$ZK_REPLICAS; i++ ))
do
    nslookup $NAME-$((i-1)).$DOMAIN 2>&1 | sed "s/^/$DATE: nslookup $NAME-$((i-1)).$DOMAIN /" >> $LOG_FILE
    echo "$DATE $?" >> $LOG_FILE 2>&1
done

# See https://serverfault.com/questions/853037/how-can-i-capture-the-exit-status-of-nc-when-run-from-timeout
ZK_CLIENT_PORT=${ZK_CLIENT_PORT:-2181}
OK=$(echo ruok | timeout --preserve-status 30 nc -w 30 127.0.0.1 $ZK_CLIENT_PORT)
DATE=`date -u '+%Y-%m-%dT%H:%M:%SZ'`
echo "$DATE Readiness or liveness probe - ruok = $OK for $HOST" >> $LOG_FILE 2>&1
if [ "$OK" == "imok" ]; then
    echo "$DATE Readiness or liveness probe succeeded for $HOST" >> $LOG_FILE 2>&1
    echo "$DATE Finished readiness or liveness probe for $HOST" >> $LOG_FILE 2>&1
    exit 0
else
    echo "$DATE Readiness or liveness probe failed for $HOST" >> $LOG_FILE 2>&1
    echo "$DATE Finished readiness or liveness probe for $HOST" >> $LOG_FILE 2>&1
    exit 1
fi